@@ -20,7 +20,9 @@ mod history;
 pub enum Msg {
     TextareaMounted(web_sys::Node),
     EditorMounted(web_sys::Node),
-    Keydown(web_sys::KeyboardEvent),
+    /// keydown from window events
+    WindowKeydown(web_sys::KeyboardEvent),
+    /// Keydown from the hidden text area
     TextareaKeydown(web_sys::KeyboardEvent),
     MoveCursor(usize, usize),
     MoveCursorToLine(usize),
@@ -49,6 +51,7 @@ pub struct Editor<XMSG> {
     #[allow(unused)]
     recorded: Recorded,
     measurements: Option<Measurements>,
+    average_update_time: Option<f64>,
     scroll_top: f32,
     scroll_left: f32,
     window_scroll_top: f32,
@@ -68,12 +71,13 @@ pub struct Editor<XMSG> {
 
 impl<XMSG> Editor<XMSG> {
     pub fn from_str(options: Options, content: &str) -> Self {
-        let editor = Editor {
+        Editor {
             options: options.clone(),
             text_buffer: TextBuffer::from_str(options, content),
             page_size: 10,
             recorded: Recorded::new(),
             measurements: None,
+            average_update_time: None,
             scroll_top: 0.0,
             scroll_left: 0.0,
             window_scroll_top: 0.0,
@@ -87,8 +91,7 @@ impl<XMSG> Editor<XMSG> {
             is_selecting: false,
             selection_start: None,
             selection_end: None,
-        };
-        editor
+        }
     }
 }
 
@@ -115,6 +118,7 @@ impl<XMSG> Component<Msg, XMSG> for Editor<XMSG> {
                 Effects::none()
             }
             Msg::TextareaInput(input) => {
+                log::trace!("text are input: {:?}", input);
                 let char_count = input.chars().count();
                 // for chrome:
                 // detect if the typed in character was a composed and becomes 1 unicode character
@@ -154,9 +158,12 @@ impl<XMSG> Component<Msg, XMSG> for Editor<XMSG> {
                     let c = key.chars().next().expect("must be only 1 chr");
                     match c {
                         'c' if is_ctrl => {
-                            //self.copy_to_clipboard();
-                            self.textarea_exec_copy();
-                            self.clear_hidden_textarea();
+                            let ret = self.command_copy();
+                            log::trace!("copy works: {}", ret);
+                        }
+                        'x' if is_ctrl => {
+                            let ret = self.command_cut();
+                            log::trace!("cut works: {}", ret);
                         }
                         'v' if is_ctrl => {
                             log::trace!("pasting is handled");
@@ -173,14 +180,16 @@ impl<XMSG> Component<Msg, XMSG> for Editor<XMSG> {
             }
             Msg::Mouseup(client_x, client_y) => {
                 let cursor = self.client_to_cursor(client_x, client_y);
+                self.command_set_position(cursor.x, cursor.y);
                 self.selection_end = Some(cursor);
                 if let (Some(start), Some(end)) =
                     (self.selection_start, self.selection_end)
                 {
                     self.is_selecting = false;
                     self.command_set_selection(start, end);
+                    // pre-emptively put the selection into the hidden textarea
+                    self.set_hidden_textarea_with_selection();
                 }
-                self.command_set_position(cursor.x, cursor.y);
 
                 if let Some(selected_text) = self.text_buffer.selected_text() {
                     log::trace!("selected text: \n{}", selected_text);
@@ -202,8 +211,10 @@ impl<XMSG> Component<Msg, XMSG> for Editor<XMSG> {
                     if let Some(start) = self.selection_start {
                         self.command_set_selection(start, cursor);
                     }
+                    Effects::none().measure()
+                } else {
+                    Effects::none().no_render()
                 }
-                Effects::none()
             }
             Msg::Paste(text_content) => {
                 self.command_insert_text(&text_content);
@@ -218,10 +229,21 @@ impl<XMSG> Component<Msg, XMSG> for Editor<XMSG> {
             Msg::EndSelection(_line, _col) => Effects::none(),
             Msg::StopSelection => Effects::none(),
             Msg::SetMeasurement(measurements) => {
+                match self.average_update_time {
+                    Some(average_update_time) => {
+                        self.average_update_time = Some(
+                            (average_update_time + measurements.total_time)
+                                / 2.0,
+                        );
+                    }
+                    None => {
+                        self.average_update_time = Some(measurements.total_time)
+                    }
+                }
                 self.measurements = Some(measurements);
                 Effects::none().no_render()
             }
-            Msg::Keydown(ke) => {
+            Msg::WindowKeydown(ke) => {
                 let key = ke.key();
                 self.process_keypresses(&ke);
                 if key.chars().count() == 1 {
@@ -282,19 +304,18 @@ impl<XMSG> Component<Msg, XMSG> for Editor<XMSG> {
                 height: 0,
                 position: "absolute",
                 padding: 0,
-                width: px(1000),
+                width: px(300),
                 height: px(0),
                 border:format!("{} solid black",px(1)),
                 bottom: units::em(-1),
-                height: units::em(1),
                 outline: "none",
             },
 
             ".hidden_textarea_wrapper": {
                 overflow: "hidden",
                 position: "relative",
-                width: px(3),
-                height: 0,
+                width: px(300),
+                height: px(0),
             },
 
             ".status": {
@@ -354,41 +375,137 @@ impl<XMSG> Editor<XMSG> {
         }
     }
 
+    /// calls on 2 ways to copy
+    /// either 1 should work
+    /// returns true if it succeded
+    fn command_copy(&self) -> bool {
+        if self.copy_to_clipboard() {
+            true
+        } else {
+            self.textarea_exec_copy()
+        }
+    }
+
+    /// try exec_cut, try cut to clipboard if the first fails
+    /// This shouldn't execute both since cut is destructive.
+    /// Returns true if it succeded
+    fn command_cut(&mut self) -> bool {
+        if self.cut_to_clipboard() {
+            true
+        } else {
+            self.textarea_exec_cut()
+        }
+    }
+
+    /// set the content of the textarea to selection
+    ///
+    /// Note: This is necessary for webkit2.
+    /// webkit2 doesn't seem to allow to fire the setting of textarea value, select and copy
+    /// in the same animation frame.
+    fn set_hidden_textarea_with_selection(&self) {
+        if let Some(selected_text) = self.text_buffer.selected_text() {
+            if let Some(ref hidden_textarea) = self.hidden_textarea {
+                log::trace!("setting the value to textarea: {}", selected_text);
+                hidden_textarea.set_value(&selected_text);
+                log::trace!("textarea value: {}", hidden_textarea.value());
+                hidden_textarea.select();
+            }
+        }
+    }
+
     /// this is for newer browsers
     /// This doesn't work on webkit2
-    #[allow(unused)]
     #[cfg(web_sys_unstable_apis)]
     #[cfg(feature = "with-navigator-clipboard")]
-    fn copy_to_clipboard(&self) {
+    fn copy_to_clipboard(&self) -> bool {
         if let Some(selected_text) = self.text_buffer.selected_text() {
             let navigator = sauron::window().navigator();
             if let Some(clipboard) = navigator.clipboard() {
                 let _ = clipboard.write_text(&selected_text);
+                return true;
+            } else {
+                log::warn!("no navigator clipboard");
             }
         }
+        false
     }
+
+    #[cfg(not(feature = "with-navigator-clipboard"))]
+    fn copy_to_clipboard(&mut self) -> bool {
+        false
+    }
+
+    #[cfg(web_sys_unstable_apis)]
+    #[cfg(feature = "with-navigator-clipboard")]
+    fn cut_to_clipboard(&mut self) -> bool {
+        if let Some(selected_text) = self.text_buffer.cut_selected_text() {
+            let navigator = sauron::window().navigator();
+            if let Some(clipboard) = navigator.clipboard() {
+                let _ = clipboard.write_text(&selected_text);
+                return true;
+            } else {
+                log::warn!("no navigator clipboard");
+            }
+        }
+        false
+    }
+
+    #[cfg(not(feature = "with-navigator-clipboard"))]
+    fn cut_to_clipboard(&mut self) -> bool {
+        false
+    }
+
     /// execute copy on the selected textarea
     /// this works even on older browser
-    fn textarea_exec_copy(&self) {
+    fn textarea_exec_copy(&self) -> bool {
         use sauron::web_sys::HtmlDocument;
 
         if let Some(selected_text) = self.text_buffer.selected_text() {
             if let Some(ref hidden_textarea) = self.hidden_textarea {
+                log::trace!("setting the value to textarea: {}", selected_text);
+                hidden_textarea.set_value(&selected_text);
+                log::trace!("textarea value: {}", hidden_textarea.value());
+                hidden_textarea.select();
+                let html_document: HtmlDocument =
+                    sauron::document().unchecked_into();
+                if let Ok(ret) = html_document.exec_command("copy") {
+                    hidden_textarea.set_value("");
+                    log::trace!("exec_copy ret: {}", ret);
+                    return ret;
+                }
+            }
+        }
+        false
+    }
+
+    /// returns true if the command succeeded
+    fn textarea_exec_cut(&mut self) -> bool {
+        use sauron::web_sys::HtmlDocument;
+
+        if let Some(selected_text) = self.text_buffer.cut_selected_text() {
+            if let Some(ref hidden_textarea) = self.hidden_textarea {
+                log::trace!("setting the value to textarea: {}", selected_text);
                 hidden_textarea.set_value(&selected_text);
 
                 hidden_textarea.select();
                 let html_document: HtmlDocument =
                     sauron::document().unchecked_into();
-                html_document.exec_command("copy").expect("must copy");
-                hidden_textarea.set_value("");
+                if let Ok(ret) = html_document.exec_command("cut") {
+                    hidden_textarea.set_value("");
+                    return ret;
+                }
             }
         }
+        false
     }
 
     fn process_keypresses(&mut self, ke: &web_sys::KeyboardEvent) {
         let key = ke.key();
         match &*key {
             "Tab" => {
+                log::trace!("tab key is pressed");
+                let tab = "    ";
+                self.text_buffer.command_insert_text(tab);
                 self.refocus_hidden_textarea();
             }
             "Enter" => {
@@ -470,9 +587,7 @@ impl<XMSG> Editor<XMSG> {
         extern_msgs
     }
 
-    ///TODO: This call the the real DOM, maybe create recalculate this everytime
-    /// a scroll even is made
-    fn editor_offset(&self) -> Option<Point2<f32>> {
+    pub fn editor_offset(&self) -> Option<Point2<f32>> {
         if let Some(ref editor_element) = self.editor_element {
             log::trace!("calculating editor offset..");
             let rect = editor_element.get_bounding_client_rect();
@@ -491,12 +606,10 @@ impl<XMSG> Editor<XMSG> {
         client_y: i32,
     ) -> Point2<i32> {
         let numberline_wide = self.text_buffer.get_numberline_wide() as f32;
-        let editor = self.editor_offset().expect("must have editor offset");
-        let col = (client_x as f32 - editor.x + self.window_scroll_left)
-            / CH_WIDTH as f32
-            - numberline_wide;
-        let line = (client_y as f32 - editor.y + self.window_scroll_top)
-            / CH_HEIGHT as f32;
+        let editor = self.editor_offset().expect("must have an editor offset");
+        let col =
+            (client_x as f32 - editor.x) / CH_WIDTH as f32 - numberline_wide;
+        let line = (client_y as f32 - editor.y) / CH_HEIGHT as f32;
         let x = col.floor() as i32;
         let y = line.floor() as i32;
         Point2::new(x, y)
@@ -619,17 +732,15 @@ impl<XMSG> Editor<XMSG> {
                         measurements.view_node_count,
                         measurements.build_view_took.round(),
                         measurements.dom_update_took.round(),
-                        measurements.total_time.round()
+                        measurements.total_time.round(),
                     )
                 } else {
                     comment("")
                 },
-                if let (Some(start), Some(end)) =
-                    (self.selection_start, self.selection_end)
-                {
-                    text!("| selection start: {} end: {}", start, end)
+                if let Some(average_update_time) = self.average_update_time {
+                    text!("| average time: {:.2}ms", average_update_time)
                 } else {
-                    text("| no selection")
+                    comment("")
                 },
             ],
         )
