@@ -94,6 +94,39 @@ impl TextBuffer {
             None
         }
     }
+
+    fn is_within_position(
+        &self,
+        (line_index, range_index, cell_index): (usize, usize, usize),
+        start: Point2<usize>,
+        end: Point2<usize>,
+    ) -> bool {
+        let x = self.lines[line_index]
+            .calc_range_cell_index_to_x(range_index, cell_index);
+        let y = line_index;
+
+        if self.options.use_block_mode {
+            x >= start.x && x <= end.x && y >= start.y && y <= end.y
+        } else {
+            if y > start.y && y < end.y {
+                true
+            } else {
+                let same_start_line = y == start.y;
+                let same_end_line = y == end.y;
+
+                if same_start_line && same_end_line {
+                    x >= start.x && x <= end.x
+                } else if same_start_line {
+                    x >= start.x
+                } else if same_end_line {
+                    x <= end.x
+                } else {
+                    false
+                }
+            }
+        }
+    }
+
     /// check if this cell is within the selection of the textarea
     pub(crate) fn in_selection(
         &self,
@@ -101,68 +134,103 @@ impl TextBuffer {
         range_index: usize,
         cell_index: usize,
     ) -> bool {
-        let x = self.lines[line_index]
-            .calc_range_cell_index_to_x(range_index, cell_index);
-        let y = line_index;
-
         if let Some((start, end)) = self.normalize_selection() {
-            if self.options.use_block_mode {
-                x >= start.x && x <= end.x && y >= start.y && y <= end.y
-            } else {
-                if y > start.y && y < end.y {
-                    true
-                } else {
-                    let same_start_line = y == start.y;
-                    let same_end_line = y == end.y;
-
-                    if same_start_line && same_end_line {
-                        x >= start.x && x <= end.x
-                    } else if same_start_line {
-                        x >= start.x
-                    } else if same_end_line {
-                        x <= end.x
-                    } else {
-                        false
-                    }
-                }
-            }
+            self.is_within_position(
+                (line_index, range_index, cell_index),
+                start,
+                end,
+            )
         } else {
             false
         }
     }
 
-    pub(crate) fn selected_text(&self) -> Option<String> {
-        if let (Some(start), Some(_end)) =
-            (self.selection_start, self.selection_end)
-        {
-            let mut buffer = TextBuffer::from_str(Options::default(), "");
-            for (line_index, line) in self.lines.iter().enumerate() {
-                let y = line_index;
-                for (range_index, range) in line.ranges.iter().enumerate() {
-                    for (cell_index, cell) in range.cells.iter().enumerate() {
-                        let x = line.calc_range_cell_index_to_x(
-                            range_index,
-                            cell_index,
-                        );
-                        if self.in_selection(
-                            line_index,
-                            range_index,
-                            cell_index,
-                        ) {
-                            if self.options.use_block_mode {
-                                buffer.insert_char(
-                                    x - start.x,
-                                    y - start.y,
-                                    cell.ch,
-                                );
+    /// TODO: build the text using the technique used in cut_text, which is more efficient way and
+    /// less code
+    pub(crate) fn get_text(
+        &self,
+        start: Point2<usize>,
+        end: Point2<usize>,
+    ) -> String {
+        let (start, end) = util::normalize_points(start, end);
+        let mut buffer = TextBuffer::from_str(Options::default(), "");
+        for (line_index, line) in self.lines.iter().enumerate() {
+            let y = line_index;
+            for (range_index, range) in line.ranges.iter().enumerate() {
+                for (cell_index, cell) in range.cells.iter().enumerate() {
+                    let x = line
+                        .calc_range_cell_index_to_x(range_index, cell_index);
+                    if self.is_within_position(
+                        (line_index, range_index, cell_index),
+                        start,
+                        end,
+                    ) {
+                        if self.options.use_block_mode {
+                            buffer.insert_char(
+                                x - start.x,
+                                y - start.y,
+                                cell.ch,
+                            );
+                        } else {
+                            //if its not in block mode, we only deduct x if it is on the first line
+                            let in_start_selection_line = y == start.y;
+                            let new_x = if in_start_selection_line {
+                                x - start.x
                             } else {
-                                buffer.insert_char(x, y - start.y, cell.ch);
-                            }
+                                x
+                            };
+                            buffer.insert_char(new_x, y - start.y, cell.ch);
                         }
                     }
                 }
             }
-            Some(buffer.to_string())
+        }
+        buffer.to_string()
+    }
+
+    /// Remove the text within the start and end position then return the deleted text
+    pub(crate) fn cut_text(
+        &mut self,
+        start: Point2<usize>,
+        end: Point2<usize>,
+    ) -> String {
+        let deleted_text = self.get_text(start, end);
+        if self.options.use_block_mode {
+            for line_index in start.y..=end.y {
+                println!("deleting cells in line: {}", line_index);
+                self.lines[line_index].delete_cells(start.x, end.x);
+            }
+        } else {
+            let is_one_line = start.y == end.y;
+            //delete the lines in between
+            if is_one_line {
+                self.lines[start.y].delete_cells(start.x, end.x);
+            } else {
+                self.lines.drain(start.y + 1..end.y);
+                // at the first line of selection: delete the cells from start.x to end
+                self.lines[start.y].delete_cells_to_end(start.x);
+                // at the last line of selection: delete the cells from 0 to end.x
+                self.lines[end.y].delete_cells_from_start(end.x);
+            }
+        }
+        deleted_text
+    }
+
+    pub(crate) fn selected_text(&self) -> Option<String> {
+        if let (Some(start), Some(end)) =
+            (self.selection_start, self.selection_end)
+        {
+            Some(self.get_text(start, end))
+        } else {
+            None
+        }
+    }
+
+    pub(crate) fn cut_selected_text(&mut self) -> Option<String> {
+        if let (Some(start), Some(end)) =
+            (self.selection_start, self.selection_end)
+        {
+            Some(self.cut_text(start, end))
         } else {
             None
         }
@@ -534,6 +602,8 @@ impl TextBuffer {
 }
 
 /// text manipulation
+/// This are purely manipulating text into the text buffer.
+/// The cursor shouldn't be move here, since it is done by the commands functions
 impl TextBuffer {
     /// the total number of lines of this text canvas
     pub(crate) fn total_lines(&self) -> usize {
@@ -581,7 +651,7 @@ impl TextBuffer {
         if let Some(line) = self.lines.get_mut(y) {
             let (range_index, col) = line
                 .calc_range_cell_index_position(x)
-                .unwrap_or(line.last_range_cell_index());
+                .unwrap_or(line.range_cell_next());
             if let Some(range_bound) = line.ranges.get_mut(range_index) {
                 range_bound.recalc_width();
                 let mut other = range_bound.split_at(col);
@@ -614,31 +684,40 @@ impl TextBuffer {
 
         let (range_index, cell_index) = self.lines[y]
             .calc_range_cell_index_position(x)
-            .unwrap_or(self.lines[y].last_range_cell_index());
+            .unwrap_or(self.lines[y].range_cell_next());
 
         self.lines[y].insert_char(range_index, cell_index, ch);
     }
 
-    pub(crate) fn insert_text(&mut self, x: usize, y: usize, text: &str) {
-        let mut new_line = y;
+    fn insert_line_text(&mut self, x: usize, y: usize, text: &str) {
         let mut new_col = x;
-        let lines: Vec<&str> = text.lines().collect();
-        let is_multi_line = lines.len() > 1;
-        if is_multi_line {
-            self.break_line(x, y);
+        for ch in text.chars() {
+            let width = ch.width().unwrap_or_else(|| {
+                panic!("must have a unicode width for {:?}", ch)
+            });
+            self.insert_char(new_col, y, ch);
+            new_col += width;
         }
-        for line in lines {
-            for ch in line.chars() {
-                let width = ch.width().unwrap_or_else(|| {
-                    panic!("must have a unicode width for {:?}", ch)
-                });
-                self.insert_char(new_col, new_line, ch);
-                new_col += width;
+    }
+
+    pub(crate) fn insert_text(&mut self, x: usize, y: usize, text: &str) {
+        self.ensure_cell_exist(x, y);
+        let lines: Vec<&str> = text.lines().collect();
+        if lines.len() == 1 {
+            self.insert_line_text(x, y, lines[0]);
+        } else {
+            dbg!(&self.lines);
+            let mut new_col = x;
+            let mut new_line = y;
+            for (line_index, line) in lines.iter().enumerate() {
+                println!("inserting {} at {},{}", line, new_col, new_line);
+                if line_index + 1 < lines.len() {
+                    self.break_line(new_col, new_line);
+                }
+                self.insert_line_text(new_col, new_line, line);
+                new_col = 0;
+                new_line += 1;
             }
-            new_col = 0;
-            new_line += 1;
-            self.break_line(new_col, new_line);
-            self.cursor.y = new_line;
         }
     }
 
@@ -691,9 +770,25 @@ impl TextBuffer {
     pub(crate) fn get_position(&self) -> Point2<usize> {
         self.cursor
     }
+
+    fn calculate_offset(&self, text: &str) -> (usize, usize) {
+        let lines: Vec<&str> = text.lines().collect();
+        let cols = if let Some(last_line) = lines.last() {
+            last_line
+                .chars()
+                .map(|ch| ch.width().expect("chars must have a width"))
+                .sum()
+        } else {
+            0
+        };
+        (cols, lines.len().saturating_sub(1))
+    }
 }
 
 /// Command implementation here
+///
+/// functions that are preceeded with command also moves the
+/// cursor and highlight the texts
 impl TextBuffer {
     pub(crate) fn command_insert_char(&mut self, ch: char) {
         self.insert_char(self.cursor.x, self.cursor.y, ch);
@@ -708,7 +803,10 @@ impl TextBuffer {
     pub(crate) fn command_insert_text(&mut self, text: &str) {
         use unicode_width::UnicodeWidthStr;
         self.insert_text(self.cursor.x, self.cursor.y, text);
-        self.move_x(text.width());
+        let (x, y) = self.calculate_offset(text);
+        self.move_y(y);
+        self.move_x(x);
+        self.calculate_focused_cell();
     }
     pub(crate) fn move_left(&mut self) {
         self.cursor.x = self.cursor.x.saturating_sub(1);
@@ -726,6 +824,10 @@ impl TextBuffer {
         self.cursor.x = self.cursor.x.saturating_add(x);
         self.calculate_focused_cell();
     }
+    pub(crate) fn move_y(&mut self, y: usize) {
+        self.cursor.y = self.cursor.y.saturating_add(y);
+        self.calculate_focused_cell();
+    }
     pub(crate) fn move_up(&mut self) {
         self.cursor.y = self.cursor.y.saturating_sub(1);
         self.calculate_focused_cell();
