@@ -1,5 +1,6 @@
 use crate::Options;
 use crate::TextBuffer;
+use nalgebra::Point2;
 
 #[test]
 fn line_length() {
@@ -9,6 +10,40 @@ fn line_length() {
     assert_eq!(buffer.line_width(0), Some(11));
 }
 
+#[test]
+fn test_get_text() {
+    let raw = "Hello world";
+    let buffer = TextBuffer::from_str(Options::default(), raw);
+    let txt = buffer.get_text(Point2::new(0, 0), Point2::new(4, 0));
+    assert_eq!(txt, "Hello");
+}
+
+#[test]
+fn test_text_selection() {
+    let raw = "Hello world";
+    let mut buffer = TextBuffer::from_str(Options::default(), raw);
+    buffer.set_selection(Point2::new(0, 0), Point2::new(4, 0));
+    let txt = buffer.selected_text();
+    assert_eq!(txt, Some("Hello".to_string()));
+}
+
+#[test]
+fn test_get_text_world() {
+    let raw = "Hello world";
+    let buffer = TextBuffer::from_str(Options::default(), raw);
+    let txt = buffer.get_text(Point2::new(6, 0), Point2::new(10, 0));
+    assert_eq!(txt, "world");
+}
+
+#[test]
+fn test_text_selection_world() {
+    let raw = "Hello world";
+    let mut buffer = TextBuffer::from_str(Options::default(), raw);
+    buffer.set_selection(Point2::new(6, 0), Point2::new(10, 0));
+    let txt = buffer.selected_text();
+    assert_eq!(txt, Some("world".to_string()));
+}
+
 #[test]
 fn delete_last_char() {
     let raw = "Hello world";
@@ -201,3 +236,73 @@ fn insert_end() {
     buffer.insert_char(5, 0, 'Y');
     assert_eq!(buffer.to_string(), "HelloY");
 }
+
+#[test]
+fn test_cut_text() {
+    let raw = "Hello world";
+    let mut buffer = TextBuffer::from_str(Options::default(), raw);
+    let txt = buffer.cut_text(Point2::new(0, 0), Point2::new(4, 0));
+    assert_eq!(txt, "Hello");
+    assert_eq!(buffer.to_string(), " world");
+}
+
+#[test]
+fn test_cut_text_multi_line() {
+    let raw = "before text\nHello world\nafter text";
+    let mut buffer = TextBuffer::from_str(Options::default(), raw);
+    let txt = buffer.cut_text(Point2::new(0, 1), Point2::new(4, 1));
+    assert_eq!(txt, "Hello");
+    assert_eq!(buffer.to_string(), "before text\n world\nafter text");
+}
+
+#[test]
+fn test_cut_text_2lines_multi_line() {
+    let raw = "before text\nHello world\nafter text";
+    let mut buffer = TextBuffer::from_str(Options::default(), raw);
+    let txt = buffer.cut_text(Point2::new(0, 1), Point2::new(4, 2));
+    assert_eq!(txt, "Hello world\nafter");
+    //FIXME: There should only be 1 \n here
+    assert_eq!(buffer.to_string(), "before text\n\n text");
+}
+
+#[test]
+fn test_cut_text_2lines_multi_line_block_mode() {
+    let raw = "before text\nHello world\nafter text";
+    let mut buffer = TextBuffer::from_str(
+        Options {
+            use_block_mode: true,
+            ..Default::default()
+        },
+        raw,
+    );
+    let txt = buffer.cut_text(Point2::new(6, 1), Point2::new(10, 2));
+    assert_eq!(txt, "world\ntext");
+    assert_eq!(buffer.to_string(), "before text\nHello \nafter ");
+}
+
+#[test]
+fn test_insert_text() {
+    let raw = "Hello world";
+    let mut buffer = TextBuffer::from_str(Options::default(), raw);
+    buffer.insert_text(5, 0, "YYYY");
+    assert_eq!(buffer.to_string(), "HelloYYYY world");
+}
+
+#[test]
+fn test_insert_multi_line_text() {
+    let raw = "Hello world";
+    let mut buffer = TextBuffer::from_str(Options::default(), raw);
+    buffer.insert_text(5, 0, "XXXX\nYYYY");
+    assert_eq!(buffer.to_string(), "HelloXXXX\nYYYY world");
+}
+
+#[test]
+fn test_insert_multi_line_text_to_multi_line_text() {
+    let raw = "before text\nHello world\nafter text";
+    let mut buffer = TextBuffer::from_str(Options::default(), raw);
+    buffer.insert_text(5, 1, "XXXX\nYYYY");
+    assert_eq!(
+        buffer.to_string(),
+        "before text\nHelloXXXX\nYYYY world\nafter text"
+    );
+}
